<?php
/*
Plugin Name: WooCommerce Adyen Payment Gateway
Plugin URI: http://codecanyon.net/item/adyen-woocommerce-payment-gateway/9575760
Description: Extends WooCommerce with Adyen payment gateway.
Version: 0.4
Author: Jaroslav Svetlik
Author URI: http://www.jprocustom.com

    Copyright:  2014 - 2018 Jaroslav Svetlik.
    License: GNU General Public License v3.0
    License URI: http://www.gnu.org/licenses/gpl-3.0.html
	Last Update: 12-10-2017
*/ 
if( ! defined( 'ABSPATH' ) ) exit;

// Include Adyen Helper Class
require_once( 'includes/class-wc-gateway-adyen-helper.php' );
 
/**
 * Initialize Adyen Gateway
 *
 * @since 0.4
 */
add_action( 'plugins_loaded', 'WC_Gateway_Adyen_init', 0 );
function WC_Gateway_Adyen_init() {
 
	if ( !class_exists( 'WC_Payment_Gateway' ) ) return;
	
    load_plugin_textdomain( 'wc-adyen', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' ); // localization
	
	/**
     * Adyen gateway class
	 * @ JproCustom_WC_Gateway_Adyen
     */
    class WC_Gateway_Adyen extends WC_Payment_Gateway {
        
        /** @var bool Whether or not sandbox is enabled */
        public static $sandbox = false;
        
        /** @var bool Whether or not logging is enabled */
        public static $log_enabled = false;

        /** @var WC_Logger Logger instance */
        public static $log = false;
		
		/**
		 * Class constructor
		 */
		function __construct() {
			$this->id 				= 'adyen';
			$this->method_title 	= __( 'Adyen', 'adyen' );
			$this->icon 			= WP_PLUGIN_URL . "/" . plugin_basename(dirname(__FILE__)) . '/assets/images/icon.png';
			$this->has_fields		= false;
			$this->debug_email 		= get_option( 'admin_email' );
			$this->liveurl			= 'https://live.adyen.com/hpp/select.shtml';
			$this->testurl			= 'https://test.adyen.com/hpp/select.shtml';
			$this->notify_url		= WC()->api_request_url( 'WC_Gateway_Adyen' );
			
			// Load the form fields.
			$this->init_form_fields();
			
			// Load the settings.
			$this->init_settings();
			
			// Define user set variables
			$this->title			= $this->settings['title'];
			$this->description		= $this->settings['description'];
			$this->merchant_account	= $this->settings['merchant_account'];
			$this->skin_code		= $this->settings['skin_code'];
			$this->hmac_key			= $this->settings['hmac_key'];
			$this->hmac_test_key	= $this->settings['hmac_test_key'];
            $this->debug            = 'yes' === $this->get_option( 'debug', 'no' );
			$this->testmode			= $this->settings['testmode'];
            $this->email            = $this->get_option( 'email' );
            $this->receiver_email   = $this->get_option( 'receiver_email', $this->email );
            
            self::$log_enabled      = $this->debug;
            self::$sandbox          = $this->testmode;
			
			// Actions
			add_action( 'valid-adyen-request', array( $this, 'successful_request' ) );
			add_action( 'woocommerce_receipt_adyen', array( $this, 'receipt_page' ) );
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
			add_action( 'woocommerce_thankyou_adyen', array( $this, 'thankyou_page' ) );
            
             // Adyen Notifications
            if( isset( $_POST['eventCode'] ) && ! empty( $_POST['eventCode'] ) ) {
                require_once( 'includes/class-wc-gateway-adyen-ipn.php' );
                new WC_Gateway_Adyen_Notifications();
            } else { // If Adyen notifications disabled, let's parse response through return url.
                // Payment listener/API hook
                add_action( 'woocommerce_api_wc_gateway_adyen', array( $this, 'check_adyen_response' ) );
            }
		}
        
        /**
         * Logging method.
         *
         * @param string $message Log message.
         * @param string $level   Optional. Default 'info'.
         *     emergency|alert|critical|error|warning|notice|info|debug
         */
        public static function log( $message, $level = 'info' ) {
            if ( self::$log_enabled ) {
                if ( empty( self::$log ) ) {
                    self::$log = wc_get_logger();
                }
                self::$log->log( $level, $message, array( 'source' => 'adyen' ) );
            }
        }
		
		/**
		 * Init Adyen Settings
		 */
		function init_form_fields() {
			$this->form_fields = include( 'includes/settings-adyen.php' );
		}
		
		/**
         * Admin Panel Options
         * - Options for bits like 'title' and availability on a country-by-country basis
         */
        public function admin_options() {
            echo '<h3>'.__( 'Adyen Payment Gateway', 'adyen' ).'</h3>';
            echo '<p>'.__( 'Adyen sustav za internet autorizaciju i naplatu kreditnih i debitnih kartica.' ).'</p>';
            echo '<table class="form-table">';
            $this->generate_settings_html();
            echo '</table>';
        }
		
		/**
         *  There are no payment fields for Adyen, but we want to show the description if set.
         */
        function payment_fields() {
            if( $this->description ) echo wpautop( wptexturize( $this->description ) );
        }
		
		/**
         * Thank you Page
         */
		function thankyou_page() {
			echo '<p>'.__( 'Thank you for shopping with us. Your account has been charged and your transaction is successful. We will be shipping your order to you soon.', 'adyen' ).'</p>';
		}
		
		/**
         * Receipt Page
         */
        function receipt_page( $order ) {
            echo '<p>'.__( 'Thank you for your order, please click the button below to pay with Adyen.', 'adyen' ).'</p>';
            echo $this->generate_adyen_form( $order );
        }
		
		/**
         * Generate Adyen button link
         */
        function generate_adyen_form( $order_id ) {
			global $woocommerce;
			
            $order = new WC_Order( $order_id );
			
			// Test mode on | off
			if( $this->testmode == 'yes' ) {
				$adyen_adr 	= $this->testurl;
				$hmacKey	= $this->hmac_test_key;
			} else {
				$adyen_adr 	= $this->liveurl;
				$hmacKey	= $this->hmac_key;
			}
			
            // WooCommerce 2.6 -> 3.0 Comaptibility
			$order_total 			= ( version_compare( WC_VERSION, '2.7', '<' ) ) ? $order->get_order_total() : $order->get_total();
            $order_total            = explode( ".", $order_total ); // Remove "." from Total Amount
			
            $merchantReference 		= $order_id.'_'.date("ymds"); // Unique order id
			$paymentAmount 			= $order_total[0].$order_total[1];
			$currencyCode 			= get_woocommerce_currency();
			$shipBeforeDate 		= date("Y-m-d",strtotime("+3 days"));
			$skinCode 				= $this->skin_code;
			$merchantAccount		= $this->merchant_account;
			$sessionValidity 		= date("c",strtotime("+1 days"));
			$shopperLocale			= 'en_US';
            
            // WooCommerce 2.6 -> 3.0 Compatibility
			$shopperEmail			= ( version_compare( WC_VERSION, '2.7', '<' ) ) ? $order->billing_email : $order->get_billing_email();

			$adyen_args = array(
				'merchantReference'			=> $merchantReference,
				'merchantAccount'			=> $merchantAccount,
				'currencyCode'				=> $currencyCode,
				'paymentAmount'				=> $paymentAmount,
				'sessionValidity'			=> $sessionValidity,
				'shipBeforeDate'			=> $shipBeforeDate,
				'shopperLocale'				=> $shopperLocale,
				'skinCode'					=> $skinCode,
				'brandCode'					=> '',
				'merchantReturnData'		=> '',
				'shopperEmail'				=> $shopperEmail,
				'shopperReference'			=> $shopperEmail
			);

            $adyen_args_array = array();
			
			// The character escape function
			$escapeval = function($val) {
			 return str_replace(':','\\:',str_replace('\\','\\\\',$val));
			};
			
			// Sort the array by key using SORT_STRING order
			ksort($adyen_args, SORT_STRING);
			
			// Generate the signing data string
			$signData = implode(":",array_map($escapeval,array_merge(array_keys($adyen_args), array_values($adyen_args))));
            
			// base64-encode the binary result of the HMAC computation
			$merchantSig = base64_encode(hash_hmac('sha256',$signData,pack("H*" , $hmacKey),true));
			$adyen_args["merchantSig"] = $merchantSig;
			
			foreach( $adyen_args as $key => $value )
			{
                $adyen_args_array[] = '<input type="hidden" name="' . htmlspecialchars($key, ENT_COMPAT | ENT_HTML401 ,'UTF-8') . '" 
															value="' . htmlspecialchars($value, ENT_COMPAT | ENT_HTML401 ,'UTF-8') . '"/>';
            }
			
			wc_enqueue_js( '
				$.blockUI({
						message: "' . esc_js( __( 'Thank you for your order. We are now redirecting you to Adyen to make payment.', 'adyen' ) ) . '",
						baseZ: 99999,
						overlayCSS:
						{
							background: "#fff",
							opacity: 0.6
						},
						css: {
							padding:        "20px",
							zindex:         "9999999",
							textAlign:      "center",
							color:          "#555",
							border:         "3px solid #aaa",
							backgroundColor:"#fff",
							cursor:         "wait",
							lineHeight:		"24px",
						}
					});
				jQuery("#submit_adyen_payment_form").click();
			' );
			
			// Return payment form
            return 	'<form action="' . esc_attr( $adyen_adr ) . '" method="POST" id="adyenForm">
					 ' . implode( '', $adyen_args_array ) . '
					 <!-- Button Fallback -->
					 <div class="payment_buttons">
						<input type="submit" class="button" id="submit_adyen_payment_form" value="'.__( 'Pay via Adyen', 'adyen' ).'" /> 
						<a class="button cancel" href="'.$order->get_cancel_order_url().'">'.__( 'Cancel order &amp; restore cart', 'adyen' ).'</a>
					 </div>
					 <script type="text/javascript">
						jQuery(".payment_buttons").hide();
					 </script>
					</form>';
        }
		
		/**
         * Process the payment and return the result
         */
        function process_payment( $order_id ) {
			
			$order = new WC_Order( $order_id );
			
			return array(
				'result' => 'success', 
				'redirect' => $order->get_checkout_payment_url( true )
			);
		}
		
		/**
		 * Function to preserve the original special character
		 *
		 * @since 0.3
		 */
		function fix($source) {
			$source = preg_replace_callback(
			  '/(^|(?<=&))[^=[&]+/',
			  function($key) {
				return bin2hex(urldecode($key[0]));
			  },
			  $source
			);
			parse_str($source, $post);
			return array_combine(array_map('hex2bin', array_keys($post)), $post);
		}
		
		/**
		 * Checking Adyen Response
		 * @ successful payment
		 * @ hacking attempt
		 * @ transaction rejected
		 */
		function check_adyen_response() {
			global $woocommerce;
			
			// Select proper hmac key
			if( $this->testmode == 'yes' ) {
				$hmacKey	= $this->hmac_test_key;
			} else {
				$hmacKey	= $this->hmac_key;
			}
			
			// Function to escape character
			function escapeval($val) {
			  return str_replace(':','\\:',str_replace('\\','\\\\',$val));
			};
			
			$return_query = isset( $_SERVER['QUERY_STRING'] ) ? $_SERVER['QUERY_STRING'] : $_REQUEST;
			
			// Lets get Adyen response parameters
			$posted = $this->fix( $return_query );

			$order_id	= (int)$posted['merchantReference'];
			$order		= new WC_Order( $order_id );
			
			// Retrieve the merchantSig from the URL parameters
			$res_merchantSig = $posted['merchantSig'];
			
			// Remove the merchantSig field for the signature calculation
			unset($posted['merchantSig']);
			
			// Sort the array by key using SORT_STRING order
			ksort($posted, SORT_STRING);
  
			// Generate the signing data string
			$signData = implode(":",array_map($escapeval,array_merge(array_keys($posted), array_values($posted))));
			
			// base64-encode the binary result of the HMAC computation
			$merchantSig = base64_encode(hash_hmac('sha256',$signData,pack("H*" , $hmacKey),true));		
			
			#############################################################
			# TRANSACTION SUCCESSFULL
			#############################################################
			if( $merchantSig == $res_merchantSig && ($posted['authResult'] == 'AUTHORISED') ) {
				$order->add_order_note( __( 'Adyen payment authorisation was successfully completed.', 'adyen' ) );
				$order->payment_complete();
                
				// Redirection to order page
				wp_redirect( $this->get_return_url($order) );
			}
			#############################################################
			# TRANSACTION REFUSED
			#############################################################
			else
			if( ($posted['authResult']) == 'REFUSED' ) {
			
				// Change the status to pending / unpaid
				$order->update_status( 'pending', __( 'Payment refused.', 'adyen' ) );
				
				// WooCommerce 2.1
				if( function_exists( 'wc_add_notice' ) ) {
					wc_add_notice( __( 'Payment #'.$order_id.' refused.'), $notice_type = 'error' );
				} else {
					// WooCommerce 2.0
					$woocommerce->add_error( __( 'Payment refused: ', 'adyen' ) . 'order ID: #'.$order_id.' refused.' );
				}
				
				wp_redirect( $woocommerce->cart->get_checkout_url());
				exit;
			}
			#############################################################
			# TRANSACTION CANCELLED
			#############################################################
			else
			if( ($posted['authResult']) == 'CANCELLED' ) {
			
				// Change the status to pending / unpaid
				$order->update_status( 'cancelled', __( 'Payment cancelled.', 'adyen' ) );
				
				// WooCommerce 2.1
				if( function_exists( 'wc_add_notice' ) ) {
					wc_add_notice( __( 'Payment #'.$order_id.' cancelled.'), $notice_type = 'error' );
				} else {
					// WooCommerce 2.0
					$woocommerce->add_error( __( 'Payment cancelled: ', 'adyen' ) . 'order ID: #'.$order_id.' cancelled.' );
				}
				
				wp_redirect( $woocommerce->cart->get_checkout_url());
				exit;
			}
			#############################################################
			# TRANSACTION PENDING
			#############################################################
			else
			if( ($posted['authResult']) == 'PENDING' ) {
			
				// Change the status to pending / unpaid
				$order->update_status( 'pending', __( 'Payment pending.', 'adyen' ) );
				
				// WooCommerce 2.1
				if( function_exists( 'wc_add_notice' ) ) {
					wc_add_notice( __( 'Payment #'.$order_id.' pending.'), $notice_type = 'error' );
				} else {
					// WooCommerce 2.0
					$woocommerce->add_error( __( 'Payment pending: ', 'adyen' ) . 'order ID: #'.$order_id.' pending.' );
				}
				
				wp_redirect( $woocommerce->cart->get_checkout_url());
				exit;
			}
			#############################################################
			# TRANSACTION ERROR
			#############################################################
			else
			if( ($posted['authResult']) == 'ERROR' ) {
			
				// Change the status to pending / unpaid
				$order->update_status( 'pending', __( 'Payment error.', 'adyen' ) );
				
				// WooCommerce 2.1
				if( function_exists( 'wc_add_notice' ) ) {
					wc_add_notice( __( 'Payment #'.$order_id.' error.'), $notice_type = 'error' );
				} else {
					// WooCommerce 2.0
					$woocommerce->add_error( __( 'Payment pending: ', 'adyen' ) . 'order ID: #'.$order_id.' error.' );
				}
				
				wp_redirect( $woocommerce->cart->get_checkout_url());
				exit;
			}
        }
	} //--> End WC_Gateway_Adyen class
	
	/**
     * Let's register Adyen Gateway into WooCommerce
     */
    function woocommerce_add_adyen_gateway( $methods ) 
	{
        $methods[] = 'WC_Gateway_Adyen';
        return $methods;
    }
	add_filter( 'woocommerce_payment_gateways', 'woocommerce_add_adyen_gateway' );
}